"""Base classes for endpoint processing"""

from urllib import urlencode
from xml.etree import ElementTree

from mstranslator.utils import Constant

__all__ = ['AUDIO', 'CONTENT', 'Endpoint', 'GetEndpoint']

AUDIO = Constant(
    MAX='MaxQuality',
    MIN='MinSize',
)

CONTENT = Constant(
    HTML='text/html',
    TXT='text/plain',
)

LANGUAGES = (
    ('ar' 'Arabic'),
    ('bg' 'Bulgarian'),
    ('ca', 'Catalan'),
    ('zh-CHS', 'Chinese (Simplified)'),
    ('zh-CHT', 'Chinese (Traditional)'),
    ('cs', 'Czech'),
    ('da', 'Danish'),
    ('nl', 'Dutch'),
    ('en', 'English'),
    ('et', 'Estonian'),
    ('fa', 'Persian (Farsi)'),
    ('fi', 'Finnish'),
    ('fr', 'French'),
    ('de', 'German'),
    ('el', 'Greek'),
    ('ht', 'Haitian Creole'),
    ('he', 'Hebrew'),
    ('hi', 'Hindi'),
    ('hu', 'Hungarian'),
    ('id', 'Indonesian'),
    ('it', 'Italian'),
    ('ja', 'Japanese'),
    ('ko', 'Korean'),
    ('lv', 'Latvian'),
    ('lt', 'Lithuanian'),
    ('mww', 'Hmong Daw'),
    ('no', 'Norwegian'),
    ('pl', 'Polish'),
    ('pt', 'Portuguese'),
    ('ro', 'Romanian'),
    ('ru', 'Russian'),
    ('sk', 'Slovak'),
    ('sl', 'Slovenian'),
    ('es', 'Spanish'),
    ('sv', 'Swedish'),
    ('th', 'Thai'),
    ('tr', 'Turkish'),
    ('uk', 'Ukrainian'),
    ('vi', 'Vietnamese '),
)

LANGUAGE_CODES = [l[0] for l in LANGUAGES]

class Endpoint(object):
    endpoint = None
    api_protocol = 'http'
    api_base = 'api.microsofttranslator.com'
    api_version = 'V2'
    api_type = 'HTTP.svc'
    params = {}
    data = None

    def convert_args(self, args):
        pass

    def convert_kwargs(self, kwargs):
        if 'self' in kwargs:
            del kwargs['self']
        if 'tfrom' in kwargs:
            kwargs['from'] = kwargs['tfrom']
            del kwargs['tfrom']

        if 'fmt' in kwargs:
            kwargs['format'] = kwargs['fmt']
            del kwargs['fmt']

        url_params = dict()

        for k, v in kwargs.items():
            url_params[self.camelize_param(k)] = v

        self.params = url_params

    def camelize_param(self, s):
        """Takes all_lowercase and turns it into camelCase"""

        final = ''
        parts = s.split('_')
        final += parts.pop(0)

        for p in parts:
            final += p[0].upper() + p[1:]

        return final

    def get_endpoint_url(self):
        if not self.endpoint:
            raise NotImplementedError('You must set endpoint attribute')

        return '%s://%s/%s/%s/%s' % (
            self.api_protocol,
            self.api_base,
            self.api_version,
            self.api_type,
            self.endpoint,
        )

    def get_quoted_params(self):
        return urlencode(self.params)

    def get_request_url(self):
        return '%s?%s' % (self.get_endpoint_url(), self.get_quoted_params())

    def store_data(self):
        raise NotImplementedError('Please override this method in subclass')

    def get_data(self):
        return self.data

    def parse_xml(self, response):
        e = ElementTree.parse(response)
        return e.getroot()

    def process_response(self, response):
        raise NotImplementedError('Please override this method in subclass')


class GetEndpoint(Endpoint):
    def store_data(self):
        raise RuntimeError('GET endpoints have no data')

    def process_response(self, response):
        """Returns verbatim response"""
        return response






from base import GetEndpoint, CONTENT, LANGUAGE_CODES

class TranslateEndpoint(GetEndpoint):
    endpoint = 'Translate'

    def __init__(self, text, tfrom, to):

        if tfrom not in LANGUAGE_CODES or to not in LANGUAGE_CODES:
            raise ValueError('Invalid language codes')

        self.convert_kwargs(locals())

    def process_response(self, resp):
        root = self.parse_xml(resp)
        return root.text



from __future__ import unicode_literals

from urllib2 import Request, urlopen, URLError
from urllib import urlencode
from datetime import datetime, timedelta
import json

from utils import Constant

from endpoints import *

AUDIO = Constant(
    MAX='MaxQuality',
    MIN='MinSize',
)

CONTENT = Constant(
    HTML='text/html',
    TXT='text/plain',
)

class MSTranslatorAccessKey(object):
    """Initializes a new access key.

    This object should only be initialized once per session. After that,
    calling the ``get_key()`` method will fetch the new key as needed.

    """

    oauth_url = 'https://datamarket.accesscontrol.windows.net/v2/OAuth2-13'
    scope_url = 'http://api.microsofttranslator.com'
    grant_type = 'client_credentials'

    created = None
    access_key = None
    expiry = 10*60

    def __init__(self, client_id, client_secret):
        self.client_id = client_id
        self.client_secret = client_secret

    def expired(self):
        if not self.created:
            return True

        return datetime.now() - self.created > timedelta(0, self.expiry)

    def create_key(self):
        self.created = datetime.now()

        data = urlencode(dict(
            client_id = self.client_id,
            client_secret = self.client_secret,
            grant_type = self.grant_type,
            scope = self.scope_url
        ))

        request = Request(url=self.oauth_url, data=data)
        response = urlopen(request)
        response = json.loads(response.read())

        self.access_key = response['access_token']
        self.expiry = response['expires_in']

        return self.access_key

    def get_key(self):
        if self.access_key and not self.expired():
            return self.access_key

        return self.create_key()


class MSTranslator(object):
    auth_header_name = 'Authorization'

    def __init__(self, api_key):
        if not isinstance(api_key, MSTranslatorAccessKey):
            raise ValueError('API key must be a MSTranslatorAccessKey instance')

        self.api_key = api_key

    # core methods

    def get_authorization_string(self):
        if not self.api_key:
            raise RuntimeError('Cannot use Translator API without API key.')

        return "Bearer %s" % self.api_key.get_key()

    def get_authorization_header(self):
        return {self.auth_header_name: self.get_authorization_string()}

    def process_endpoint(self, endpoint):
        request = Request(
            url=endpoint.get_request_url(),
            headers=self.get_authorization_header(),
        )

        data=endpoint.get_data()

        if data:
            request.add_data(data)

        return endpoint.process_response(urlopen(request))

    # api calls

    def detect(self, text):
        """Detect language used in the text and return the language code.

        """

        raise NotImplementedError('detect is not implemented yet')

    def detect_array(self, texts):
        """Detect langauges used in texts, and return a list of codes.

        """

        raise NotImplementedError('detect_array is not implemented yet')

    def get_languages_for_translation(self):
        """Get a list of language codes supported by translate feature.

        """

        raise NotImplementedError('get_languages_for_translate is not '
                                  'implemented yet')

    def translate(self, *args, **kwargs):
        """Translate text from one language to another.

        """
        endpoint = TranslateEndpoint(*args, **kwargs)
        return self.process_endpoint(endpoint)

